//This file is part of the Java Compiler Kit (JKit)

//The Java Compiler Kit is free software; you can 
//redistribute it and/or modify it under the terms of the 
//GNU General Public License as published by the Free Software 
//Foundation; either version 2 of the License, or (at your 
//option) any later version.

//The Java Compiler Kit is distributed in the hope
//that it will be useful, but WITHOUT ANY WARRANTY; without 
//even the implied warranty of MERCHANTABILITY or FITNESS FOR 
//A PARTICULAR PURPOSE.  See the GNU General Public License 
//for more details.

//You should have received a copy of the GNU General Public 
//License along with the Java Compiler Kit; if not, 
//write to the Free Software Foundation, Inc., 59 Temple Place, 
//Suite 330, Boston, MA  02111-1307  USA

//(C) David James Pearce, 2009. 

package jkit;

import java.io.*;
import jkit.compiler.SyntaxError;
import jkit.java.io.*;

public class Main {

	/**
	 * Main method provides command-line processing capability.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		if(!new Main().compile(args)) {	
			System.exit(1);
		} else {
			System.exit(0);
		}
	}
	
	public boolean compile(String[] args) {
		boolean verbose = true;
		
		try {
			// ======================================================
			// ============== Third, load skeletons ================
			// ======================================================		

			
			for(int i=0;i!=args.length;++i) {
				String filename = args[i];
				JavaFileReader jfr = new JavaFileReader(filename);
				// jfile is the syntax tree
				JavaFile jfile = jfr.read();
				

				String outname = "output" + File.separatorChar + filename.replaceAll("\\.[^.]*\\z", ".java");
				System.out.println("Writing: " + outname);
								
				File of = new File(outname);
				of.getParentFile().mkdirs();
				
				FileWriter fout = new FileWriter(of);				
				JavaFileWriter jwr = new JavaFileWriter(fout); //strips ownership
				SourceFileWriter swr = new SourceFileWriter(fout); //does nothing
				TransFileWriter twr = new TransFileWriter(fout);  //translates to Java
				
				twr.write(jfile);				
				fout.close();
			}
			
		} catch (SyntaxError e) {
			outputSourceError(e.fileName(), e.line(), e.column(), e.width(), e
					.getMessage());
			if (verbose) {
				e.printStackTrace(System.err);
			}
			return false;
		} catch(Exception e) {
			System.err.println("Error: " + e.getMessage());
			if (verbose) {
				e.printStackTrace(System.err);
			}
			return false;
		}
		
		return true;
	}
		

	public static void outputSourceError(String fileArg, int line, int col,
			int width, String message) {
		System.err.println("Error in file: " + fileArg + ", line: " + line + ", " + message);
		return;
		
//		String l = readLine(fileArg, line);		
//		if(l != null) {
//			System.err.println(l);
//			for (int j = 0; j < Math.min(col,l.length()); ++j) {
//				if (l.charAt(j) == '\t') {
//					System.err.print("\t");
//				} else {
//					System.err.print(" ");
//				}
//			}		
//			for (int j = 0; j < width; ++j)
//				System.err.print("^");
//			System.err.println("");
//		} else {
//			// We shouldn't be able to get here. But, if there is a bug in jkit
//			// itself, such that it attributes the wrong filename to the file
//			// containing the error, then we can. So, it's helpful not to throw
//			// an exception at this point ...
//		}
	}

	public static String readLine(String f, int l) {
		try {
			return readLine(new FileReader(f), l);
		} catch (IOException e) {
			// shouldn't get here.
			return "";
		}
	}

	public static String readLine(Reader in, int l) {
		try {
			LineNumberReader lin = new LineNumberReader(in);
			String line = "";
			while (lin.getLineNumber() < l && line != null) {
				line = lin.readLine();
			}
			return line;
		} catch (IOException e) {
			return "";
		}
	}
}
